/* global jest describe beforeEach it expect */

'use strict';

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _reactAddons = require('react/addons');

var _reactAddons2 = _interopRequireDefault(_reactAddons);

jest.dontMock('../components/Card');
jest.dontMock('../utils/inject-style');

var TestUtils = _reactAddons2['default'].addons.TestUtils;

// Babel would move an import in front of the jest.dontMock. That's why require
// is used instead of import.
var Card = require('../components/Card');

describe('Card', function () {
  it('should come with default styles', function () {
    var card = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(Card, null));
    var divNode = TestUtils.findRenderedDOMComponentWithTag(card, 'div');

    expect(divNode.props.style).toBeDefined();
    expect(divNode.props.style.background).toBe('#fff');
  });

  it('should be able to adopt the style of the card', function () {
    var card = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(Card, { style: { background: '#F00' } }));
    var divNode = TestUtils.findRenderedDOMComponentWithTag(card, 'div');

    expect(divNode.props.style.background).toEqual('#F00');
  });

  it('should render its children', function () {
    var card = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      Card,
      null,
      _reactAddons2['default'].createElement(
        'span',
        null,
        'Hello there'
      )
    ));
    var divNode = TestUtils.findRenderedDOMComponentWithTag(card, 'div');
    var spanNode = TestUtils.findRenderedDOMComponentWithTag(card, 'span');

    expect(divNode.props.children.type).toEqual('span');
    expect(spanNode.props.children).toEqual('Hello there');
  });
});